/* Complete code as a header file */

#include <cassert>
#include <vector>
#include <pthread.h>
#include <boost/type_traits.hpp>
#include <boost/tuple/tuple.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>
#include <errno.h>
#include <sys/errno.h>
#include "spawn_tools.h"

namespace detail 
{
  struct thread_description_base
  {
    private:
      pthread_t             pt;
      mutable volatile bool was_joined;
      mutable boost::mutex  join_mutex;

    public:
      thread_description_base () : was_joined (false) {};

      virtual ~thread_description_base (); // implemented below

      void create (void * (*p) (void *), void *d) {
        pthread_create (&pt, 0, p, d);
      };
        
      void join () const {
        if (was_joined)
          return;

        boost::mutex::scoped_lock lock(join_mutex);
        if (!was_joined)
          pthread_join (pt, 0);
        was_joined = true;
      };
  };

  template <typename RT>
  struct thread_description : public thread_description_base
  {
      return_value<RT> ret_val;
  };

  thread_description_base::~thread_description_base () 
  {
                                     // if we are here, then the
                                     // last listener is just about
                                     // to cease to exists. there
                                     // are two possibilities (since
                                     // there are two types of
                                     // possible listeners, and
                                     // exactly one listener is
                                     // around):
                                     //
                                     // 1. there are no Thread<>
                                     // objects around any more, but
                                     // we are at the end of the
                                     // entry_point function of the
                                     // new thread; the thread is
                                     // still running, though, and
                                     // we are called from this new
                                     // thread.
                                     //
                                     // if this is the case,
                                     // apparently none of the
                                     // orginal listeners has called
                                     // join(), otherwise the thread
                                     // would not be running any
                                     // more. since there are no
                                     // listeners any more, nobody
                                     // will ever call join() on
                                     // this thread any
                                     // more. however, to avoid a
                                     // resource leak, somebody has
                                     // to do something like that --
                                     // on the other hand, we can't
                                     // since calling pthread_join()
                                     // on ourself will create a
                                     // deadlock, of course. so
                                     // detach the thread.
                                     //
                                     // note that in this case,
                                     // since nobody has ever called
                                     // join(), was_joined must be
                                     // false
                                     //
                                     // 2. the thread has already
                                     // ended. in this case, the
                                     // last Thread<> object
                                     // refering to it just went out
                                     // of scope. If this is the
                                     // case, to prevent a memory
                                     // leak, we have to call join
                                     // on the thread, if this has
                                     // not yet happened, i.e. if
                                     // was_joined is false.
                                     //
                                     // if we are in case 2, then
                                     // the destructor is called
                                     // from another than the
                                     // presently running thread
                                     //
                                     //
                                     // now: how do we find out
                                     // which case we're in? we
                                     // could either call
                                     // pthread_join and see whether
                                     // we get an error back, or if
                                     // we get back an error on
                                     // pthread_detach. from the man
                                     // pages, the second way seems
                                     // a little bit safer, so go it:
    if (was_joined == false)
      {
                                         // assume case 1:
        int error = pthread_detach (pt);
        if (error == 0)
          return;

                                         // ouch, could not
                                         // detach. see if thread
                                         // could not be found any
                                         // more:
        if (error == ESRCH)
                                           // ok, this is the
                                           // case. then we are in
                                           // branch 2, and need to
                                           // join the thread
          join ();
        else
                                           // something went
                                           // terribly wrong
          abort ();
      }
  }
}



template <typename RT = void>
class thread {
  public:
    thread () {};
    thread (const boost::shared_ptr<detail::thread_description<RT> > &td)
                    : thread_descriptor (td) {};    
    
    void join () const { thread_descriptor->join (); };

    RT return_value () {
      join ();
      return thread_descriptor->ret_val.get();
    };

    bool operator == (const thread &t) {
      return thread_descriptor == t.thread_descriptor;
    };
    
  private:
    boost::shared_ptr<detail::thread_description<RT> > thread_descriptor;
};



template <typename RT = void>
class thread_group 
{
  public:
    thread_group & operator += (const thread<RT> &t) {
      threads.push_back (t);
      return *this;
    };

    void join_all () const {
      for (typename std::vector<thread<RT> >::const_iterator
             t=threads.begin(); t!=threads.end(); ++t)
        t->join ();
    };
    
  private:
    std::vector<thread<RT> > threads;
};



namespace detail
{

  template <typename RT>
  struct wrapper_base {
      typedef void * (*EntryPoint) (void *);

      thread<RT> fire_up () {
        thread_descriptor
          = DescriptionPointer(new detail::thread_description<RT>());

        boost::mutex::scoped_lock lock (mutex);        
        thread_descriptor->create (entry_point, (void *)this);
        condition.wait (lock);

        return thread_descriptor;
      }

    protected:
      wrapper_base (const EntryPoint ep)
                      : entry_point (ep) {};

      typedef boost::shared_ptr<detail::thread_description<RT> >
      DescriptionPointer;
      
      DescriptionPointer thread_descriptor;

      mutable boost::mutex     mutex;    
      mutable boost::condition condition;

    private:
      const EntryPoint entry_point;
  };
  


  template <typename RT, typename ArgList>
  struct fun_wrapper : public wrapper_base<RT> {
      typedef typename detail::fun_ptr<RT,ArgList>::type FunPtr;
      typedef typename detail::tie_args<ArgList>::type ArgReferences;

      fun_wrapper (FunPtr               fun_ptr,
                   const ArgReferences &args)
                      : wrapper_base<RT> (&entry_point),
                        fun_ptr (fun_ptr),
                        args (args)  {};
    private:
      fun_wrapper ();
      fun_wrapper (const fun_wrapper &);
      
      FunPtr        fun_ptr;
      ArgReferences args;
      
      static void * entry_point (void *arg)
        {
          const wrapper_base<RT> *w
            = reinterpret_cast<const wrapper_base<RT>*> (arg);
          const fun_wrapper *wrapper
            = static_cast<const fun_wrapper*> (w);
          FunPtr    fun_ptr = wrapper->fun_ptr;
          ArgList   args    = wrapper->args;

          boost::shared_ptr<detail::thread_description<RT> >
            thread_descriptor  = wrapper->thread_descriptor;
          
          {
            boost::mutex::scoped_lock lock (wrapper->mutex);
            wrapper->condition.notify_one ();
          }
          
          detail::call (fun_ptr, args,
                        thread_descriptor->ret_val);
          
          return 0;
        };
  };

  
  
  template <typename RT, class C, typename ArgList>
  struct mem_fun_wrapper : public wrapper_base<RT> {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      
      typedef typename detail::tie_args<ArgList>::type ArgReferences;
      
      mem_fun_wrapper (MemFunPtr            mem_fun_ptr,
                       C                   &c,
                       const ArgReferences &args)
                      : wrapper_base<RT> (&entry_point),
                        c (c),
                        mem_fun_ptr (mem_fun_ptr),
                        args (args)  {};
    private:
      mem_fun_wrapper ();
      mem_fun_wrapper (const mem_fun_wrapper &);
      
      C            &c;
      MemFunPtr     mem_fun_ptr;
      ArgReferences args;
      
      static void * entry_point (void *arg)
        {
          const wrapper_base<RT> *w
            = reinterpret_cast<const wrapper_base<RT>*> (arg);
          const mem_fun_wrapper *wrapper
            = static_cast<const mem_fun_wrapper*> (w);
          MemFunPtr mem_fun_ptr = wrapper->mem_fun_ptr;
          C        &c           = wrapper->c;
          ArgList   args        = wrapper->args;

          boost::shared_ptr<detail::thread_description<RT> >
            thread_descriptor  = wrapper->thread_descriptor;
          
          {
            boost::mutex::scoped_lock lock (wrapper->mutex);
            wrapper->condition.notify_one ();
          }
          
          detail::call (mem_fun_ptr, c, args,
                        thread_descriptor->ret_val);
          
          return 0;
        };
  };
}



namespace detail
{
                                   // forward declaration of function
                                   // encapsulator
  template <typename RT, typename ArgList,
            int length = boost::tuples::length<ArgList>::value>
  class fun_encapsulator;


                                   // forward declaration of member
                                   // function encapsulator
  template <typename RT, typename C, typename ArgList,
            int length = boost::tuples::length<ArgList>::value>
  class mem_fun_encapsulator;
}


// ----------- encapsulators for member functions not taking any parameters

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with no arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 0>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() () {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              ArgList()).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}

 
                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * no arguments.
                                  */
template <typename RT, typename C>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<> >
spawn (C &c, RT (C::*fun_ptr)()) {
  return detail::mem_fun_encapsulator<RT, C, boost::tuple<> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with no
                                  * arguments.
                                  */
template <typename RT, typename C>
inline
detail::mem_fun_encapsulator<RT,const C,boost::tuple<> >
spawn (const C &c, RT (C::*fun_ptr)() const) {
  return detail::mem_fun_encapsulator<RT, const C, boost::tuple<> > (c,fun_ptr);
}




// ----------- encapsulators for unary member functions

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 1 argument.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 1>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}

 
                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 1 argument.
                                  */
template <typename RT, typename C, typename Arg1>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1> >
spawn (C &c, RT (C::*fun_ptr)(Arg1)) {
  return detail::mem_fun_encapsulator<RT, C, boost::tuple<Arg1> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 1
                                  * argument.
                                  */
template <typename RT, typename C, typename Arg1>
inline
detail::mem_fun_encapsulator<RT,const C,boost::tuple<Arg1> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1) const) {
  return detail::mem_fun_encapsulator<RT, const C, boost::tuple<Arg1> > (c,fun_ptr);
}




// ----------- encapsulators for binary member functions

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 2 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 2>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,
                                                         arg2)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 2 arguments.
                                  */
template <typename RT, typename C, typename Arg1, typename Arg2>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2)) {
  return detail::mem_fun_encapsulator<RT, C, boost::tuple<Arg1, Arg2> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 2
                                  * arguments.
                                  */
template <typename RT, typename C, typename Arg1, typename Arg2>
inline
detail::mem_fun_encapsulator<RT,const C,boost::tuple<Arg1, Arg2> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2) const) {
  return detail::mem_fun_encapsulator<RT, const C, boost::tuple<Arg1, Arg2> > (c,fun_ptr);
}



// ----------- encapsulators for ternary member functions

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 3 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 3>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,
                                                         arg2,
                                                         arg3)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 3 arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2, Arg3> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3)) {
  return detail::mem_fun_encapsulator<RT, C,
    boost::tuple<Arg1, Arg2, Arg3> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 3
                                  * arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3>
inline
detail::mem_fun_encapsulator<RT,const C,boost::tuple<Arg1, Arg2, Arg3> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3) const) {
  return detail::mem_fun_encapsulator<RT, const C,
    boost::tuple<Arg1, Arg2, Arg3> > (c,fun_ptr);
}




// ----------- encapsulators for member functions with 4 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 4 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 4>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,arg2,
                                                         arg3,arg4)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 4 arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3, typename Arg4>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2, Arg3, Arg4> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4)) {
  return detail::mem_fun_encapsulator<RT, C,
    boost::tuple<Arg1, Arg2, Arg3, Arg4> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 4
                                  * arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3, typename Arg4>
inline
detail::mem_fun_encapsulator<RT,const C,boost::tuple<Arg1, Arg2, Arg3, Arg4> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4) const) {
  return detail::mem_fun_encapsulator<RT, const C,
    boost::tuple<Arg1, Arg2, Arg3, Arg4> > (c,fun_ptr);
}




// ----------- encapsulators for member functions with 5 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 5 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 5>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,arg2,
                                                         arg3,arg4,
                                                         arg5)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 5 arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2, Arg3, Arg4, Arg5> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5)) {
  return detail::mem_fun_encapsulator<RT, C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 5
                                  * arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5>
inline
detail::mem_fun_encapsulator<RT,const C,boost::tuple<Arg1, Arg2, Arg3, Arg4, Arg5> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5) const) {
  return detail::mem_fun_encapsulator<RT, const C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5> > (c,fun_ptr);
}


// ----------- encapsulators for member functions with 6 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 6 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 6>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,arg2,
                                                         arg3,arg4,
                                                         arg5,arg6)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 6 arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6)) {
  return detail::mem_fun_encapsulator<RT, C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 6
                                  * arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6>
inline
detail::mem_fun_encapsulator<RT,const C,
                             boost::tuple<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6) const) {
  return detail::mem_fun_encapsulator<RT, const C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6> > (c,fun_ptr);
}


// ----------- encapsulators for member functions with 7 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 7 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 7>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6,
                  typename boost::tuples::element<6,ArgList>::type arg7) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,arg2,
                                                         arg3,arg4,
                                                         arg5,arg6,
                                                         arg7)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 7 arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2, Arg3,
                                               Arg4, Arg5, Arg6, Arg7> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7)) {
  return detail::mem_fun_encapsulator<RT, C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 7
                                  * arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7>
inline
detail::mem_fun_encapsulator<RT,const C,
                             boost::tuple<Arg1, Arg2, Arg3,
                                          Arg4, Arg5, Arg6, Arg7> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7) const) {
  return detail::mem_fun_encapsulator<RT, const C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7> > (c,fun_ptr);
}


// ----------- encapsulators for member functions with 8 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 8 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 8>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6,
                  typename boost::tuples::element<6,ArgList>::type arg7,
                  typename boost::tuples::element<7,ArgList>::type arg8) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,arg2,
                                                         arg3,arg4,
                                                         arg5,arg6,
                                                         arg7,arg8)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 8 arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2, Arg3,
                                               Arg4, Arg5, Arg6,
                                               Arg7, Arg8> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                              Arg6,Arg7,Arg8)) {
  return detail::mem_fun_encapsulator<RT, C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 8
                                  * arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8>
inline
detail::mem_fun_encapsulator<RT,const C,
                             boost::tuple<Arg1, Arg2, Arg3,
                                          Arg4, Arg5, Arg6,
                                          Arg7, Arg8> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                                    Arg6,Arg7,Arg8) const) {
  return detail::mem_fun_encapsulator<RT, const C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8> > (c,fun_ptr);
}


// ----------- encapsulators for member functions with 9 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 9 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 9>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6,
                  typename boost::tuples::element<6,ArgList>::type arg7,
                  typename boost::tuples::element<7,ArgList>::type arg8,
                  typename boost::tuples::element<8,ArgList>::type arg9) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,arg2,
                                                         arg3,arg4,
                                                         arg5,arg6,
                                                         arg7,arg8,
                                                         arg9)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 9 arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8, typename Arg9>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2, Arg3,
                                               Arg4, Arg5, Arg6,
                                               Arg7, Arg8, Arg9> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                              Arg6,Arg7,Arg8,Arg9)) {
  return detail::mem_fun_encapsulator<RT, C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8, Arg9> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 9
                                  * arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8, typename Arg9>
inline
detail::mem_fun_encapsulator<RT,const C,
                             boost::tuple<Arg1, Arg2, Arg3,
                                          Arg4, Arg5, Arg6,
                                          Arg7, Arg8, Arg9> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                                    Arg6,Arg7,Arg8,Arg9) const) {
  return detail::mem_fun_encapsulator<RT, const C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8, Arg9> > (c,fun_ptr);
}


// ----------- encapsulators for member functions with 10 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for member
                                    * functions with 10 arguments.
                                    */
  template <typename RT, typename C, typename ArgList>
  class mem_fun_encapsulator<RT, C, ArgList, 10>
  {
      typedef typename detail::mem_fun_ptr<RT,C,ArgList>::type MemFunPtr;      

    public:
      inline mem_fun_encapsulator (C &c, MemFunPtr mem_fun_ptr)
                      : c (c), mem_fun_ptr(mem_fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6,
                  typename boost::tuples::element<6,ArgList>::type arg7,
                  typename boost::tuples::element<7,ArgList>::type arg8,
                  typename boost::tuples::element<8,ArgList>::type arg9,
                  typename boost::tuples::element<9,ArgList>::type arg10) {
        return mem_fun_wrapper<RT,C,ArgList> (mem_fun_ptr, c,
                                              boost::tie(arg1,arg2,
                                                         arg3,arg4,
                                                         arg5,arg6,
                                                         arg7,arg8,
                                                         arg9, arg10)).fire_up ();
      };
    
    private:
      C         &c;
      MemFunPtr  mem_fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the non-const spawn
                                  * function for member functions with
                                  * 10 arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8, typename Arg9,
          typename Arg10>
inline
detail::mem_fun_encapsulator<RT,C,boost::tuple<Arg1, Arg2, Arg3,
                                               Arg4, Arg5, Arg6,
                                               Arg7, Arg8, Arg9, Arg10> >
spawn (C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                              Arg6,Arg7,Arg8,Arg9,Arg10)) {
  return detail::mem_fun_encapsulator<RT, C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8, Arg9,
    Arg10> > (c,fun_ptr);
}

                                 /**
                                  * Overload of the spawn function for
                                  * const member functions with 10
                                  * arguments.
                                  */
template <typename RT, typename C,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8, typename Arg9,
          typename Arg10>
inline
detail::mem_fun_encapsulator<RT,const C,
                             boost::tuple<Arg1, Arg2, Arg3,
                                          Arg4, Arg5, Arg6,
                                          Arg7, Arg8, Arg9, Arg10> >
spawn (const C &c, RT (C::*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                                    Arg6,Arg7,Arg8,Arg9,Arg10) const) {
  return detail::mem_fun_encapsulator<RT, const C,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8, Arg9,
    Arg10> > (c,fun_ptr);
}



// ----------- encapsulators for functions not taking any parameters

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with no arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 0>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() () {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        ArgList()).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}

 
                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with no arguments.
                                  */
template <typename RT>
inline
detail::fun_encapsulator<RT,boost::tuple<> >
spawn (RT (*fun_ptr)()) {
  return detail::fun_encapsulator<RT, boost::tuple<> > (fun_ptr);
}




// ----------- encapsulators for unary functions

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 1 argument.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 1>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}

 
                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 1 argument.
                                  */
template <typename RT, typename Arg1>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1> >
spawn (RT (*fun_ptr)(Arg1)) {
  return detail::fun_encapsulator<RT, boost::tuple<Arg1> > (fun_ptr);
}




// ----------- encapsulators for binary functions

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 2 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 2>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,
                                                   arg2)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 2 arguments.
                                  */
template <typename RT, typename Arg1, typename Arg2>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2> >
spawn (RT (*fun_ptr)(Arg1,Arg2)) {
  return detail::fun_encapsulator<RT, boost::tuple<Arg1, Arg2> > (fun_ptr);
}



// ----------- encapsulators for ternary functions

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 3 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 3>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,
                                                   arg2,
                                                   arg3)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 3 arguments.
                                  */
template <typename RT,
          typename Arg1, typename Arg2, typename Arg3>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2, Arg3> >
spawn (RT (*fun_ptr)(Arg1,Arg2,Arg3)) {
  return detail::fun_encapsulator<RT,
    boost::tuple<Arg1, Arg2, Arg3> > (fun_ptr);
}




// ----------- encapsulators for functions with 4 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 4 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 4>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,arg2,
                                                   arg3,arg4)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 4 arguments.
                                  */
template <typename RT,
          typename Arg1, typename Arg2, typename Arg3, typename Arg4>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2, Arg3, Arg4> >
spawn (RT (*fun_ptr)(Arg1,Arg2,Arg3,Arg4)) {
  return detail::fun_encapsulator<RT,
    boost::tuple<Arg1, Arg2, Arg3, Arg4> > (fun_ptr);
}




// ----------- encapsulators for functions with 5 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 5 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 5>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,arg2,
                                                   arg3,arg4,
                                                   arg5)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 5 arguments.
                                  */
template <typename RT,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2, Arg3, Arg4, Arg5> >
spawn (RT (*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5)) {
  return detail::fun_encapsulator<RT,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5> > (fun_ptr);
}


// ----------- encapsulators for functions with 6 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 6 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 6>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,arg2,
                                                   arg3,arg4,
                                                   arg5,arg6)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 6 arguments.
                                  */
template <typename RT,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2, Arg3, Arg4, Arg5, Arg6> >
spawn (RT (*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6)) {
  return detail::fun_encapsulator<RT,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6> > (fun_ptr);
}


// ----------- encapsulators for functions with 7 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 7 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 7>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6,
                  typename boost::tuples::element<6,ArgList>::type arg7) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,arg2,
                                                   arg3,arg4,
                                                   arg5,arg6,
                                                   arg7)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 7 arguments.
                                  */
template <typename RT,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2, Arg3,
                                         Arg4, Arg5, Arg6, Arg7> >
spawn (RT (*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7)) {
  return detail::fun_encapsulator<RT,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7> > (fun_ptr);
}


// ----------- encapsulators for functions with 8 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 8 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 8>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6,
                  typename boost::tuples::element<6,ArgList>::type arg7,
                  typename boost::tuples::element<7,ArgList>::type arg8) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,arg2,
                                                   arg3,arg4,
                                                   arg5,arg6,
                                                   arg7,arg8)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 8 arguments.
                                  */
template <typename RT,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2, Arg3,
                                         Arg4, Arg5, Arg6,
                                         Arg7, Arg8> >
spawn (RT (*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                     Arg6,Arg7,Arg8)) {
  return detail::fun_encapsulator<RT,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8> > (fun_ptr);
}


// ----------- encapsulators for functions with 9 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 9 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 9>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6,
                  typename boost::tuples::element<6,ArgList>::type arg7,
                  typename boost::tuples::element<7,ArgList>::type arg8,
                  typename boost::tuples::element<8,ArgList>::type arg9) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,arg2,
                                                   arg3,arg4,
                                                   arg5,arg6,
                                                   arg7,arg8,
                                                   arg9)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 9 arguments.
                                  */
template <typename RT,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8, typename Arg9>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2, Arg3,
                                         Arg4, Arg5, Arg6,
                                         Arg7, Arg8, Arg9> >
spawn (RT (*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                     Arg6,Arg7,Arg8,Arg9)) {
  return detail::fun_encapsulator<RT,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8, Arg9> > (fun_ptr);
}


// ----------- encapsulators for functions with 10 arguments

namespace detail
{
                                   /**
                                    * Encapsulator class for
                                    * functions with 10 arguments.
                                    */
  template <typename RT, typename ArgList>
  class fun_encapsulator<RT, ArgList, 10>
  {
      typedef typename detail::fun_ptr<RT,ArgList>::type MemFunPtr;      

    public:
      inline fun_encapsulator (MemFunPtr fun_ptr)
                      : fun_ptr(fun_ptr) {};

      inline
      thread<RT>
      operator() (typename boost::tuples::element<0,ArgList>::type arg1,
                  typename boost::tuples::element<1,ArgList>::type arg2,
                  typename boost::tuples::element<2,ArgList>::type arg3,
                  typename boost::tuples::element<3,ArgList>::type arg4,
                  typename boost::tuples::element<4,ArgList>::type arg5,
                  typename boost::tuples::element<5,ArgList>::type arg6,
                  typename boost::tuples::element<6,ArgList>::type arg7,
                  typename boost::tuples::element<7,ArgList>::type arg8,
                  typename boost::tuples::element<8,ArgList>::type arg9,
                  typename boost::tuples::element<9,ArgList>::type arg10) {
        return fun_wrapper<RT,ArgList> (fun_ptr,
                                        boost::tie(arg1,arg2,
                                                   arg3,arg4,
                                                   arg5,arg6,
                                                   arg7,arg8,
                                                   arg9, arg10)).fire_up ();
      };
    
    private:
      MemFunPtr  fun_ptr;
  };
  
}


                                 /**
                                  * Overload of the spawn function for
                                  * non-member or static member
                                  * functions with 10 arguments.
                                  */
template <typename RT,
          typename Arg1, typename Arg2, typename Arg3,
          typename Arg4, typename Arg5, typename Arg6,
          typename Arg7, typename Arg8, typename Arg9,
          typename Arg10>
inline
detail::fun_encapsulator<RT,boost::tuple<Arg1, Arg2, Arg3,
                                         Arg4, Arg5, Arg6,
                                         Arg7, Arg8, Arg9, Arg10> >
spawn (RT (*fun_ptr)(Arg1,Arg2,Arg3,Arg4,Arg5,
                     Arg6,Arg7,Arg8,Arg9,Arg10)) {
  return detail::fun_encapsulator<RT,
    boost::tuple<Arg1, Arg2, Arg3,
    Arg4, Arg5, Arg6,
    Arg7, Arg8, Arg9,
    Arg10> > (fun_ptr);
}


