namespace detail 
{
                                   /**
                                    * A type that is used to
                                    * distinguish argument lists of
                                    * functions by enumeration.
                                    */
  template <int> struct int2type {};
} 


namespace detail 
{
                                   /**
                                    * Given an arbitrary type RT,
                                    * store an element of it and grant
                                    * access to it through functions
                                    * get() and set(). There are
                                    * specializations for reference
                                    * types (which cannot be set), and
                                    * for type void.
                                    */
  template <typename RT> struct return_value
  {
    private:
      RT value;
    public:
      inline return_value () : value() {}
      
      inline RT get () const { return value; }
      inline void set (RT v) { value = v; }
  };

  
                                   /**
                                    * Given an arbitrary type RT,
                                    * store an element of it and grant
                                    * access to it through functions
                                    * get() and set(). This is the
                                    * specialization for reference
                                    * types: since they cannot be set
                                    * after construction time, we
                                    * store a pointer instead, that
                                    * holds the address of the object
                                    * being referenced.
                                    */
  template <typename RT> struct return_value<RT &>
  {
    private:
      RT * value;
    public:
      inline return_value () : value(0) {}

      inline RT & get () const { return *value; }
      inline void set (RT & v) { value = &v; }
  };

  
                                   /**
                                    * Given an arbitrary type RT,
                                    * store an element of it and grant
                                    * access to it through functions
                                    * get() and set(). This is the
                                    * specialization for type void:
                                    * there is obviously nothing to
                                    * store, so no function set(), and
                                    * a function get() that returns
                                    * void.
                                    */
  template <> struct return_value<void> {
      static inline void get () {}
  };
}

  

namespace detail
{
                                   /**
                                    * Call arbitrary functions with
                                    * return type RT. For each number
                                    * of arguments to these functions,
                                    * there is an instance of the
                                    * do_call function in this class
                                    * that unpacks the argument list
                                    * (which is passed by reference)
                                    * and calls the function. The
                                    * number of arguments is
                                    * distinguished by the last
                                    * argument. The return value
                                    * object is the second last. A
                                    * second version of the do_call
                                    * function is used to call member
                                    * function pointers, in which case
                                    * there is an additional argument
                                    * at second position holding a
                                    * reference to the object with
                                    * which the member function
                                    * pointer is to be called.
                                    *
                                    * There is a specialization of
                                    * this class for the case that the
                                    * return type is void, in which
                                    * case there is no return value to
                                    * be set.
                                    */
  template <typename RT>
  struct Caller 
  {
                                       /**
                                        * Call a function with 0
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<0> &)
        {
          ret_val.set ((*fun_ptr) ());
        }

                                       /**
                                        * Call a member function with
                                        * 0 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<0> &)
        {
          ret_val.set ((obj.*fun_ptr) ());
        }


                                       /**
                                        * Call a function with 1
                                        * argument, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<1> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>()));
        }

                                       /**
                                        * Call a member function with
                                        * 1 argument, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<1> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>()));
        }




                                       /**
                                        * Call a function with 2
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<2> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>()));
        }

                                       /**
                                        * Call a member function with
                                        * 2 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<2> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>()));
        }


                                       /**
                                        * Call a function with 3
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<3> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>(),
                                   arg_list.template get<2>()));
        }

                                       /**
                                        * Call a member function with
                                        * 3 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<3> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>(),
                                       arg_list.template get<2>()));
        }


                                       /**
                                        * Call a function with 4
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<4> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>(),
                                   arg_list.template get<2>(),
                                   arg_list.template get<3>()));
        }

                                       /**
                                        * Call a member function with
                                        * 4 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<4> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>(),
                                       arg_list.template get<2>(),
                                       arg_list.template get<3>()));
        }


                                       /**
                                        * Call a function with 5
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<5> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>(),
                                   arg_list.template get<2>(),
                                   arg_list.template get<3>(),
                                   arg_list.template get<4>()));
        }

                                       /**
                                        * Call a member function with
                                        * 5 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<5> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>(),
                                       arg_list.template get<2>(),
                                       arg_list.template get<3>(),
                                       arg_list.template get<4>()));
        }


                                       /**
                                        * Call a function with 6
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<6> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>(),
                                   arg_list.template get<2>(),
                                   arg_list.template get<3>(),
                                   arg_list.template get<4>(),
                                   arg_list.template get<5>()));
        }

                                       /**
                                        * Call a member function with
                                        * 6 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<6> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>(),
                                       arg_list.template get<2>(),
                                       arg_list.template get<3>(),
                                       arg_list.template get<4>(),
                                       arg_list.template get<5>()));
        }


                                       /**
                                        * Call a function with 7
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<7> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>(),
                                   arg_list.template get<2>(),
                                   arg_list.template get<3>(),
                                   arg_list.template get<4>(),
                                   arg_list.template get<5>(),
                                   arg_list.template get<6>()));
        }

                                       /**
                                        * Call a member function with
                                        * 7 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<7> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>(),
                                       arg_list.template get<2>(),
                                       arg_list.template get<3>(),
                                       arg_list.template get<4>(),
                                       arg_list.template get<5>(),
                                       arg_list.template get<6>()));
        }


                                       /**
                                        * Call a function with 8
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<8> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>(),
                                   arg_list.template get<2>(),
                                   arg_list.template get<3>(),
                                   arg_list.template get<4>(),
                                   arg_list.template get<5>(),
                                   arg_list.template get<6>(),
                                   arg_list.template get<7>()));
        }

                                       /**
                                        * Call a member function with
                                        * 8 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<8> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>(),
                                       arg_list.template get<2>(),
                                       arg_list.template get<3>(),
                                       arg_list.template get<4>(),
                                       arg_list.template get<5>(),
                                       arg_list.template get<6>(),
                                       arg_list.template get<7>()));
        }


                                       /**
                                        * Call a function with 9
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<9> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>(),
                                   arg_list.template get<2>(),
                                   arg_list.template get<3>(),
                                   arg_list.template get<4>(),
                                   arg_list.template get<5>(),
                                   arg_list.template get<6>(),
                                   arg_list.template get<7>(),
                                   arg_list.template get<8>()));
        }

                                       /**
                                        * Call a member function with
                                        * 9 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<9> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>(),
                                       arg_list.template get<2>(),
                                       arg_list.template get<3>(),
                                       arg_list.template get<4>(),
                                       arg_list.template get<5>(),
                                       arg_list.template get<6>(),
                                       arg_list.template get<7>(),
                                       arg_list.template get<8>()));
        }


                                       /**
                                        * Call a function with 10
                                        * arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<10> &)
        {
          ret_val.set ((*fun_ptr) (arg_list.template get<0>(),
                                   arg_list.template get<1>(),
                                   arg_list.template get<2>(),
                                   arg_list.template get<3>(),
                                   arg_list.template get<4>(),
                                   arg_list.template get<5>(),
                                   arg_list.template get<6>(),
                                   arg_list.template get<7>(),
                                   arg_list.template get<8>(),
                                   arg_list.template get<9>()));
        }

                                       /**
                                        * Call a member function with
                                        * 10 arguments, and set the
                                        * return value.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<RT> &ret_val,
                                  const int2type<10> &)
        {
          ret_val.set ((obj.*fun_ptr) (arg_list.template get<0>(),
                                       arg_list.template get<1>(),
                                       arg_list.template get<2>(),
                                       arg_list.template get<3>(),
                                       arg_list.template get<4>(),
                                       arg_list.template get<5>(),
                                       arg_list.template get<6>(),
                                       arg_list.template get<7>(),
                                       arg_list.template get<8>(),
                                       arg_list.template get<9>()));
        }
  };



  
                                   /**
                                    * Call arbitrary functions with
                                    * void return type. For each
                                    * number of arguments to these
                                    * functions, there is an instance
                                    * of the do_call function in this
                                    * class that unpacks the argument
                                    * list (which is passed by
                                    * reference) and calls the
                                    * function. The number of
                                    * arguments is distinguished by
                                    * the last argument. The return
                                    * value object is the second last,
                                    * but since the return type is
                                    * void, this is of course simply
                                    * ignored. A second version of the
                                    * do_call function is used to call
                                    * member function pointers, in
                                    * which case there is an
                                    * additional argument at second
                                    * position holding a reference to
                                    * the object with which the member
                                    * function pointer is to be
                                    * called.
                                    */
  template <>
  struct Caller<void>
  {
                                       /**
                                        * Call a void function with 0
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &,
                                  detail::return_value<void> &,
                                  const int2type<0> &)
        {
          (*fun_ptr) ();
        }

                                       /**
                                        * Call a void member function
                                        * with 0 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &,
                                  detail::return_value<void> &,
                                  const int2type<0> &)
        {
          (obj.*fun_ptr) ();
        }


                                       /**
                                        * Call a void function with 1
                                        * argument.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<1> &)
        {
          (*fun_ptr) (arg_list.template get<0>());
        }

                                       /**
                                        * Call a void member function
                                        * with 1 argument.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<1> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>());
        }




                                       /**
                                        * Call a void function with 2
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<2> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>());
        }

                                       /**
                                        * Call a void member function
                                        * with 2 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<2> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>());
        }


                                       /**
                                        * Call a void function with 3
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<3> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>(),
                      arg_list.template get<2>());
        }

                                       /**
                                        * Call a void member function
                                        * with 3 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<3> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>(),
                          arg_list.template get<2>());
        }


                                       /**
                                        * Call a void function with 4
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<4> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>(),
                      arg_list.template get<2>(),
                      arg_list.template get<3>());
        }

                                       /**
                                        * Call a void member function
                                        * with 4 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<4> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>(),
                          arg_list.template get<2>(),
                          arg_list.template get<3>());
        }


                                       /**
                                        * Call a void function with 5
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<5> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>(),
                      arg_list.template get<2>(),
                      arg_list.template get<3>(),
                      arg_list.template get<4>());
        }

                                       /**
                                        * Call a void member function
                                        * with 5 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<5> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>(),
                          arg_list.template get<2>(),
                          arg_list.template get<3>(),
                          arg_list.template get<4>());
        }


                                       /**
                                        * Call a void function with 6
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<6> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>(),
                      arg_list.template get<2>(),
                      arg_list.template get<3>(),
                      arg_list.template get<4>(),
                      arg_list.template get<5>());
        }

                                       /**
                                        * Call a void member function
                                        * with 6 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<6> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>(),
                          arg_list.template get<2>(),
                          arg_list.template get<3>(),
                          arg_list.template get<4>(),
                          arg_list.template get<5>());
        }


                                       /**
                                        * Call a void function with 7
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<7> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>(),
                      arg_list.template get<2>(),
                      arg_list.template get<3>(),
                      arg_list.template get<4>(),
                      arg_list.template get<5>(),
                      arg_list.template get<6>());
        }

                                       /**
                                        * Call a void member function
                                        * with 7 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<7> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>(),
                          arg_list.template get<2>(),
                          arg_list.template get<3>(),
                          arg_list.template get<4>(),
                          arg_list.template get<5>(),
                          arg_list.template get<6>());
        }


                                       /**
                                        * Call a void function with 8
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<8> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>(),
                      arg_list.template get<2>(),
                      arg_list.template get<3>(),
                      arg_list.template get<4>(),
                      arg_list.template get<5>(),
                      arg_list.template get<6>(),
                      arg_list.template get<7>());
        }

                                       /**
                                        * Call a void member function
                                        * with 8 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<8> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>(),
                          arg_list.template get<2>(),
                          arg_list.template get<3>(),
                          arg_list.template get<4>(),
                          arg_list.template get<5>(),
                          arg_list.template get<6>(),
                          arg_list.template get<7>());
        }


                                       /**
                                        * Call a void function with 9
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<9> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>(),
                      arg_list.template get<2>(),
                      arg_list.template get<3>(),
                      arg_list.template get<4>(),
                      arg_list.template get<5>(),
                      arg_list.template get<6>(),
                      arg_list.template get<7>(),
                      arg_list.template get<8>());
        }

                                       /**
                                        * Call a void member function
                                        * with 9 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<9> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>(),
                          arg_list.template get<2>(),
                          arg_list.template get<3>(),
                          arg_list.template get<4>(),
                          arg_list.template get<5>(),
                          arg_list.template get<6>(),
                          arg_list.template get<7>(),
                          arg_list.template get<8>());
        }


                                       /**
                                        * Call a void function with 10
                                        * arguments.
                                        */
      template <typename PFun, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<10> &)
        {
          (*fun_ptr) (arg_list.template get<0>(),
                      arg_list.template get<1>(),
                      arg_list.template get<2>(),
                      arg_list.template get<3>(),
                      arg_list.template get<4>(),
                      arg_list.template get<5>(),
                      arg_list.template get<6>(),
                      arg_list.template get<7>(),
                      arg_list.template get<8>(),
                      arg_list.template get<9>());
        }

                                       /**
                                        * Call a void member function
                                        * with 10 arguments.
                                        */
      template <typename PFun, typename C, typename ArgList>
      static inline void do_call (PFun     fun_ptr,
                                  C       &obj,
                                  ArgList &arg_list,
                                  detail::return_value<void> &,
                                  const int2type<10> &)
        {
          (obj.*fun_ptr) (arg_list.template get<0>(),
                          arg_list.template get<1>(),
                          arg_list.template get<2>(),
                          arg_list.template get<3>(),
                          arg_list.template get<4>(),
                          arg_list.template get<5>(),
                          arg_list.template get<6>(),
                          arg_list.template get<7>(),
                          arg_list.template get<8>(),
                          arg_list.template get<9>());
        }
  };
  
  

                                   /**
                                    * Call an arbitrary function by
                                    * dispatching to the functions in
                                    * the Caller class based on the
                                    * number of elements in the
                                    * argument list and the return
                                    * type.
                                    */
  template <typename RT, typename PFun, typename ArgList>
  static inline void call (PFun     fun_ptr,
                           ArgList &arg_list,
                           detail::return_value<RT> &ret_val)
  {
    Caller<RT>::do_call (fun_ptr, arg_list, ret_val,
                         int2type<boost::tuples::length<ArgList>::value>());
  }


  
                                   /**
                                    * Call an arbitrary member
                                    * function by dispatching to the
                                    * functions in the Caller class
                                    * based on the number of elements
                                    * in the argument list and the
                                    * return type.
                                    */
  template <typename RT, typename PFun, typename C, typename ArgList>
  static inline void call (PFun     fun_ptr,
                           C       &obj,
                           ArgList &arg_list,
                           detail::return_value<RT> &ret_val)
  {
    Caller<RT>::do_call (fun_ptr, obj, arg_list, ret_val,
                         int2type<boost::tuples::length<ArgList>::value>());
  }  
}



namespace detail
{
                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments, and whether the
                                    * second argument is a const or
                                    * non-const class, dependening on
                                    * which the member function will
                                    * also me const or
                                    * non-const. There are
                                    * specializations of this class
                                    * for each number of arguments,
                                    * and the const and non-const
                                    * versions.
                                    */
  template <typename RT, class C, typename ArgList,
            int length = boost::tuples::length<ArgList>::value>
  struct mem_fun_ptr_helper;
  

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 0 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 0>
  {
      typedef RT (C::*type) ();
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 0 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 0>
  {
      typedef RT (C::*type) () const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 1 argument
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 1>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 1 argument
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 1>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type) const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 2 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 2>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 2 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 2>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type) const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 3 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 3>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 3 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 3>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type) const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 4 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 4>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 4 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 4>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type) const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 5 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 5>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 5 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 5>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type) const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 6 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 6>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 6 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 6>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type) const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 7 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 7>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type,
                             typename boost::tuples::element<6,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 7 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 7>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type,
                             typename boost::tuples::element<6,ArgList>::type) const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 8 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 8>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type,
                             typename boost::tuples::element<6,ArgList>::type,
                             typename boost::tuples::element<7,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 8 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 8>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type,
                             typename boost::tuples::element<6,ArgList>::type,
                             typename boost::tuples::element<7,ArgList>::type) const;
  };


                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 9 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 9>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type,
                             typename boost::tuples::element<6,ArgList>::type,
                             typename boost::tuples::element<7,ArgList>::type,
                             typename boost::tuples::element<8,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 9 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 9>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type,
                             typename boost::tuples::element<6,ArgList>::type,
                             typename boost::tuples::element<7,ArgList>::type,
                             typename boost::tuples::element<8,ArgList>::type) const;
  };



                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 10 arguments
                                    * and non-const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, C, ArgList, 10>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type,
                             typename boost::tuples::element<6,ArgList>::type,
                             typename boost::tuples::element<7,ArgList>::type,
                             typename boost::tuples::element<8,ArgList>::type,
                             typename boost::tuples::element<9,ArgList>::type);
  };

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 10 arguments
                                    * and const member functions.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr_helper<RT, const C, ArgList, 10>
  {
      typedef RT (C::*type) (typename boost::tuples::element<0,ArgList>::type,
                             typename boost::tuples::element<1,ArgList>::type,
                             typename boost::tuples::element<2,ArgList>::type,
                             typename boost::tuples::element<3,ArgList>::type,
                             typename boost::tuples::element<4,ArgList>::type,
                             typename boost::tuples::element<5,ArgList>::type,
                             typename boost::tuples::element<6,ArgList>::type,
                             typename boost::tuples::element<7,ArgList>::type,
                             typename boost::tuples::element<8,ArgList>::type,
                             typename boost::tuples::element<9,ArgList>::type) const;
  };

  

                                   /**
                                    * Construct a pointer to member
                                    * function based on the template
                                    * arguments, and whether the
                                    * second argument is a const or
                                    * non-const class, dependening on
                                    * which the member function will
                                    * also me const or non-const. We
                                    * do this by dispatching to the
                                    * mem_fun_ptr_helper classes that
                                    * are overloaded on the number of
                                    * elements and the const/non-const
                                    * decision.
                                    *
                                    * Note that the last template
                                    * argument for the
                                    * mem_fun_ptr_helper class is
                                    * automatically computed in the
                                    * default argument to the general
                                    * template.
                                    */
  template <typename RT, class C, typename ArgList>
  struct mem_fun_ptr
  {
      typedef typename mem_fun_ptr_helper<RT,C,ArgList>::type type;
  };  
}


namespace detail
{
                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments, and whether the
                                    * second argument is a const or
                                    * non-const class, dependening on
                                    * which the member function will
                                    * also me const or
                                    * non-const. There are
                                    * specializations of this class
                                    * for each number of arguments.
                                    */
  template <typename RT, typename ArgList,
            int length = boost::tuples::length<ArgList>::value>
  struct fun_ptr_helper;
  

                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 0 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 0>
  {
      typedef RT (*type) ();
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 1 argument.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 1>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type);
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 2 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 2>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type);
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 3 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 3>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type,
                          typename boost::tuples::element<2,ArgList>::type);
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 4 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 4>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type,
                          typename boost::tuples::element<2,ArgList>::type,
                          typename boost::tuples::element<3,ArgList>::type);
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 5 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 5>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type,
                          typename boost::tuples::element<2,ArgList>::type,
                          typename boost::tuples::element<3,ArgList>::type,
                          typename boost::tuples::element<4,ArgList>::type);
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 6 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 6>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type,
                          typename boost::tuples::element<2,ArgList>::type,
                          typename boost::tuples::element<3,ArgList>::type,
                          typename boost::tuples::element<4,ArgList>::type,
                          typename boost::tuples::element<5,ArgList>::type);
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 7 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 7>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type,
                          typename boost::tuples::element<2,ArgList>::type,
                          typename boost::tuples::element<3,ArgList>::type,
                          typename boost::tuples::element<4,ArgList>::type,
                          typename boost::tuples::element<5,ArgList>::type,
                          typename boost::tuples::element<6,ArgList>::type);
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 8 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 8>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type,
                          typename boost::tuples::element<2,ArgList>::type,
                          typename boost::tuples::element<3,ArgList>::type,
                          typename boost::tuples::element<4,ArgList>::type,
                          typename boost::tuples::element<5,ArgList>::type,
                          typename boost::tuples::element<6,ArgList>::type,
                          typename boost::tuples::element<7,ArgList>::type);
  };


                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 9 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 9>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type,
                          typename boost::tuples::element<2,ArgList>::type,
                          typename boost::tuples::element<3,ArgList>::type,
                          typename boost::tuples::element<4,ArgList>::type,
                          typename boost::tuples::element<5,ArgList>::type,
                          typename boost::tuples::element<6,ArgList>::type,
                          typename boost::tuples::element<7,ArgList>::type,
                          typename boost::tuples::element<8,ArgList>::type);
  };



                                   /**
                                    * Construct a pointer to non-member
                                    * function based on the template
                                    * arguments. This is the
                                    * specialization for 10 arguments.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr_helper<RT, ArgList, 10>
  {
      typedef RT (*type) (typename boost::tuples::element<0,ArgList>::type,
                          typename boost::tuples::element<1,ArgList>::type,
                          typename boost::tuples::element<2,ArgList>::type,
                          typename boost::tuples::element<3,ArgList>::type,
                          typename boost::tuples::element<4,ArgList>::type,
                          typename boost::tuples::element<5,ArgList>::type,
                          typename boost::tuples::element<6,ArgList>::type,
                          typename boost::tuples::element<7,ArgList>::type,
                          typename boost::tuples::element<8,ArgList>::type,
                          typename boost::tuples::element<9,ArgList>::type);
  };

  

                                   /**
                                    * Construct a pointer to
                                    * non-member function based on the
                                    * template arguments. We do this
                                    * by dispatching to the
                                    * fun_ptr_helper classes that are
                                    * overloaded on the number of
                                    * elements.
                                    *
                                    * Note that the last template
                                    * argument for the
                                    * fun_ptr_helper class is
                                    * automatically computed in the
                                    * default argument to the general
                                    * template.
                                    */
  template <typename RT, typename ArgList>
  struct fun_ptr
  {
      typedef typename fun_ptr_helper<RT,ArgList>::type type;
  };  
}


namespace detail 
{
                                   /**
                                    * Extract the Nth element of the
                                    * type list and make it a
                                    * reference.
                                    */
  template <int N, typename Tuple>
  struct add_reference_to_Nth
  {
      typedef typename boost::tuples::element<N,Tuple>::type ArgType;
      typedef typename boost::add_reference<ArgType>::type type;
  };
  
                                   /**
                                    * Specializations of this template
                                    * declare a typedef to a tuple
                                    * type that has the same basic
                                    * types as the first template
                                    * argument, but all references
                                    * instead of values. The second
                                    * argument is used to distinguish
                                    * between the lengths of argument
                                    * lists. The default argument
                                    * makes it possible to omit this
                                    * length argument.
                                    */
  template <typename Tuple, int = boost::tuples::length<Tuple>::value>
  struct tie_args_helper;


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 0.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,0>
  {
      typedef Tuple type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 1.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,1>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type>
      type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 2.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,2>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type>
      type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 3.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,3>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type,
                   typename add_reference_to_Nth<2,Tuple>::type>
      type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 4.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,4>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type,
                   typename add_reference_to_Nth<2,Tuple>::type,
                   typename add_reference_to_Nth<3,Tuple>::type>
      type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 5.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,5>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type,
                   typename add_reference_to_Nth<2,Tuple>::type,
                   typename add_reference_to_Nth<3,Tuple>::type,
                   typename add_reference_to_Nth<4,Tuple>::type>
      type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 6.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,6>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type,
                   typename add_reference_to_Nth<2,Tuple>::type,
                   typename add_reference_to_Nth<3,Tuple>::type,
                   typename add_reference_to_Nth<4,Tuple>::type,
                   typename add_reference_to_Nth<5,Tuple>::type>
      type;
  };



                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 7.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,7>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type,
                   typename add_reference_to_Nth<2,Tuple>::type,
                   typename add_reference_to_Nth<3,Tuple>::type,
                   typename add_reference_to_Nth<4,Tuple>::type,
                   typename add_reference_to_Nth<5,Tuple>::type,
                   typename add_reference_to_Nth<6,Tuple>::type>
      type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 8.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,8>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type,
                   typename add_reference_to_Nth<2,Tuple>::type,
                   typename add_reference_to_Nth<3,Tuple>::type,
                   typename add_reference_to_Nth<4,Tuple>::type,
                   typename add_reference_to_Nth<5,Tuple>::type,
                   typename add_reference_to_Nth<6,Tuple>::type,
                   typename add_reference_to_Nth<7,Tuple>::type>
      type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 9.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,9>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type,
                   typename add_reference_to_Nth<2,Tuple>::type,
                   typename add_reference_to_Nth<3,Tuple>::type,
                   typename add_reference_to_Nth<4,Tuple>::type,
                   typename add_reference_to_Nth<5,Tuple>::type,
                   typename add_reference_to_Nth<6,Tuple>::type,
                   typename add_reference_to_Nth<7,Tuple>::type,
                   typename add_reference_to_Nth<8,Tuple>::type>
      type;
  };


                                   /**
                                    * Make a tuple type of all
                                    * references out of the given
                                    * tuple. Specialization for tuple
                                    * of length 10.
                                    */
  template <typename Tuple>
  struct tie_args_helper<Tuple,10>
  {
      typedef 
      boost::tuple<typename add_reference_to_Nth<0,Tuple>::type,
                   typename add_reference_to_Nth<1,Tuple>::type,
                   typename add_reference_to_Nth<2,Tuple>::type,
                   typename add_reference_to_Nth<3,Tuple>::type,
                   typename add_reference_to_Nth<4,Tuple>::type,
                   typename add_reference_to_Nth<5,Tuple>::type,
                   typename add_reference_to_Nth<6,Tuple>::type,
                   typename add_reference_to_Nth<7,Tuple>::type,
                   typename add_reference_to_Nth<8,Tuple>::type,
                   typename add_reference_to_Nth<9,Tuple>::type>
      type;
  };


  
                                   /**
                                    * Declare a typedef to a tuple
                                    * type that has the same basic
                                    * types as the template
                                    * argument, but all references
                                    * instead of values.
                                    *
                                    * Do so by redirecting to the
                                    * tie_args_helper specializations;
                                    * note that the second argument of
                                    * these templates is computed
                                    * automatically by the default
                                    * argument specification.
                                    */
  template <typename Tuple>
  struct tie_args 
  {
      typedef typename tie_args_helper<Tuple>::type type;
  };
}
